require "infopark_component_cache/guards/cms_state_guard"

module InfoparkComponentCache
  module Guards
    # @author Tomasz Przedmojski <tomasz.przedmojski@infopark.de>
    #
    # This guard ensures that any object, whose valid until date has been passed
    # will cause inconsistency.
    class ValidUntil < CmsStateGuard
      def consistent?
        if min_valid_until_known?
          no_changes_since?
        else
          current_min_valid_until.nil?
        end
      end

      def guard!
        cache.write(cache_key, current_min_valid_until)
      end

      # @return true if a timestamp can be read from cache with {#cache_key}
      def min_valid_until_known?
        cache.exist?(cache_key) && cache.read(cache_key).kind_of?(Time)
      end

      # @return true if no obj has been changed since last {#guard!}
      def no_changes_since?
        current_min_valid_until == cache.read(cache_key) && current_min_valid_until > Time.now
      end

      # @return [String] the cache key for storing {#current_min_valid_until}
      def cache_key
        component.cache_key("min_valid_until")
      end

      # @return [Time] the timestamp of the the object that will be deactivated in nearest future
      def current_min_valid_until
        str_value = scoped_relation.where("valid_until > ?", Time.now.to_iso).minimum(:valid_until)
        return str_value if str_value.kind_of? Time

        RailsConnector::DateAttribute.parse(str_value) if str_value.present?
      end
    end
  end
end
